#include "block_data.h"

const char *block_names[BLOCK_MAX] =
{
    "Air",
    "Stone",
    "Sand",
    "Wood",
    "Leaves",
    "Grass",
    "Planks",
    "Wall",
    "Coal Ore"
};

COLOR block_colors[BLOCK_MAX];

//Maps location in texture atlas to block ID
BLOCK texture_atlas[][16] =
{
    { 0, BLOCK_STONE, BLOCK_DIRT, 0, BLOCK_PLANKS, 0, 0, BLOCK_WALL, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, BLOCK_SAND, 0, BLOCK_WOOD, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, BLOCK_COAL_ORE, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, BLOCK_LEAVES, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
};

TextureAtlasEntry *block_textures = nullptr;

void init_blockData(TEXTURE *texture)
{
    block_textures = new TextureAtlasEntry[sizeof(texture_atlas) / sizeof(BLOCK)];
    int fields_x = sizeof(*texture_atlas)/sizeof(**texture_atlas);
    int fields_y = sizeof(texture_atlas)/sizeof(*texture_atlas);
    int field_width = texture->width / fields_x;
    int field_height = texture->height / fields_y;

    int pixels = field_width * field_height;
    for(int y = 0; y < fields_y; y++)
        for(int x = 0; x < fields_x; x++)
        {
            if(texture_atlas[y][x] == 0)
                continue;

            block_textures[texture_atlas[y][x]] = textureArea(x * field_width, y * field_height, field_width, field_height);

            //Get an average color of the block
            RGB sum;
            for(int tex_x = x * field_width; tex_x < x * field_width + field_width; tex_x++)
                for(int tex_y = y * field_width; tex_y < y * field_width + field_width; tex_y++)
                {
                    RGB rgb = rgbColor(texture->bitmap[tex_x + tex_y*texture->width]);
                    sum.r += rgb.r;
                    sum.g += rgb.g;
                    sum.b += rgb.b;
                }

            sum.r /= pixels;
            sum.g /= pixels;
            sum.b /= pixels;

            block_colors[texture_atlas[y][x]] = colorRGB(sum);
        }
}

void uninit_blockData()
{
    delete[] block_textures;
}
